"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ServiceContainerImp = void 0;
let guard = {
    notNull: function (name) {
        if (name == null) {
            throw new Error('argument is null');
        }
    },
    notFound: function (name) {
        throw new Error(name + " could not found in context");
    }
};
class ServiceContainerImp {
    constructor() {
        this.settings = {};
    }
    add(name, type) {
        this.settings[name] = { type, scope: 'scoped', args: [] };
        return new ServiceCollectionBuilderImp(this.settings[name]);
    }
    addWith(name, mapfn) {
        this.settings[name] = { func: mapfn, scope: 'scoped', args: [] };
        return new ServiceCollectionBuilderImp(this.settings[name]);
    }
    use(fn) {
        fn(this);
        return this;
    }
    /**
     * @returns {ServiceProvider}
     */
    build() {
        let o = new ServiceProviderImp(this.settings);
        Object.keys(this.settings).forEach((key) => {
            Object.defineProperty(o, key, {
                get() {
                    return o.get(key);
                },
                set(newValue) {
                }
            });
        });
        return o;
    }
}
exports.ServiceContainerImp = ServiceContainerImp;
class ServiceCollectionBuilderImp {
    constructor(setting) {
        this.setting = setting;
        this.setting.scope = 'scoped';
    }
    /**
     * @returns {ServiceCollectionBuilder}
     */
    scoped() {
        this.setting.scope = 'scoped';
        return this;
    }
    /**
     * @returns {ServiceCollectionBuilder}
     */
    transient() {
        this.setting.scope = 'transient';
        return this;
    }
    /**
     * @returns {ServiceCollectionBuilder}
     */
    singleton() {
        this.setting.scope = 'singleton';
        return this;
    }
    /**
     * @returns {ServiceCollectionBuilder}
     */
    args(...args) {
        this.setting.args = args;
        return this;
    }
}
class ServiceProviderImp {
    constructor(settings) {
        this.__settings = settings;
        this.__settings["service.provider"] = { instance: this, args: [] };
        Object.keys(this.__settings)
            .filter(key => this.__settings[key].scope === 'singleton')
            .forEach((singleton) => {
            this.get(singleton);
        });
    }
    get(name, scope) {
        guard.notNull(name);
        let node = this.__settings[name];
        scope = scope || {};
        if (node == null)
            guard.notFound(name);
        if (node.instance != null) { //for singleton
            return node.instance;
        }
        if (node.scope === 'transient' && scope[name] != null) { // for transient
            return scope[name];
        }
        let instance;
        if (node.type != null) {
            let args = node.args.map(arg => this.get(arg, scope));
            if (typeof node.type === "function") {
                instance = new (Function.bind.apply(node.type, [null].concat(args)));
            }
            else {
                instance = node.type; // constant
            }
        }
        if (node.func != null) {
            if (typeof node.func != 'function') {
                throw new Error('[' + name + ']:node.func is not function');
            }
            instance = node.func(this);
        }
        if (node.scope === 'singleton') {
            node.instance = instance;
        }
        if (node.scope === 'transient') {
            scope[name] = instance;
        }
        return instance;
    }
    clone() {
        let o = new ServiceProviderImp(this.__settings);
        Object.keys(this.__settings).forEach((key) => {
            Object.defineProperty(o, key, {
                get() {
                    return o.get(key);
                },
                set(newValue) {
                }
            });
        });
        return o;
    }
}
