"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.schema = exports.yubiql = void 0;
const utility_1 = require("../query/utility");
const yubiql_1 = require("../query/yubiql");
const model_1 = require("../schema/model");
function yubiql() {
    return function (req, res, next) {
        req.yubi = {};
        req.yubi.ql = function () {
            if (req.body == null)
                return [];
            if (utility_1.isObject(req.body)) {
                let items = [];
                Object.keys(req.body).forEach((key) => {
                    let value = req.body[key];
                    switch (key) {
                        case '$filter':
                            items.push(yubiql_1.filter(value));
                            break;
                        case '$first':
                            items.push(yubiql_1.first(value));
                            break;
                        case '$sort':
                            items.push(yubiql_1.sort(value));
                            break;
                        case '$skip':
                            items.push(yubiql_1.skip(value));
                            break;
                        case '$take':
                            items.push(yubiql_1.take(value));
                            break;
                        case '$count':
                            items.push(yubiql_1.count());
                            break;
                        case '$select':
                            items.push(yubiql_1.select(value));
                            break;
                        default:
                            throw new Error(key + " not support");
                    }
                });
                return items;
            }
            return [];
        };
        res.to = function (response) {
            try {
                return finalize(req, res, next, response);
            }
            catch (error) {
                next(error);
            }
        };
        next();
    };
}
exports.yubiql = yubiql;
function responseQuery(req, queryable) {
    return __awaiter(this, void 0, void 0, function* () {
        let ql = req.yubi.ql();
        let resQ = ql.length == 0 ? queryable : queryable.query.apply(queryable, ql);
        return yield resQ.memorize();
    });
}
function sendOk(res, data = undefined) {
    res.status(200).send(data);
}
function finalize(req, res, next, result) {
    if (result == null) {
        sendOk(res);
        return;
    }
    if (result instanceof Promise) {
        result.then((r) => {
            if (r != null) {
                return finalize(req, res, next, r);
            }
            sendOk(res, r);
        }, e => next(e));
        return;
    }
    if (typeof result["query"] === "function") {
        responseQuery(req, result).then((r) => {
            sendOk(res, r);
        }, e => next(e));
        return;
    }
    sendOk(res, result);
}
function schema(model) {
    return function (req, res, next) {
        let options = (function () {
            if (req.provider != null) {
                try {
                    return req.provider.get('model-options');
                }
                finally {
                    return model_1.getOptions();
                }
            }
            return model_1.getOptions();
        })();
        let result = model_1.forModel(model, options).validate(req.body);
        if (result.isValid) {
            req.model = result.value;
            next();
        }
        else {
            req.model = result.value;
            res.status(400).send(result.error);
        }
    };
}
exports.schema = schema;
