export declare function iterate(fn: () => Iterator<any>): Iterable<any>;
export interface Group<TKey, T> extends EnumerableIterator<T> {
    key: TKey;
}
export interface ObjectDic<T> {
    [key: string]: T;
}
export declare class Enumerable<T> {
    constructor(items: Iterable<T>);
    static sum<T>(items: Iterable<T>, selector: (elem: T) => number): number;
    static contains<T>(items: Iterable<T>, element: T, selector?: (pre: T, current: T) => boolean): boolean;
    static filter<T>(items: Iterable<T>, predicate: (elem: T) => boolean): Iterable<any>;
    static repeat<T>(element: T | ((index: number) => T), count?: number, reset?: number): EnumerableIterator<T>;
    static times(count: number, action: (index: number) => void): void;
    static map<T, TResult>(items: Iterable<T>, selector: (elem: T) => TResult): Iterable<any>;
    static distinct<T>(items: Iterable<T>, equalSelector?: (pre: T, current: T) => boolean): Iterable<any>;
    static reduce<T>(items: Iterable<T>, selector: (pre: T, next: T) => T): T;
    static count(items: Iterable<any>): number;
    static extract<T>(items: Iterable<T>, extractItems: Iterable<T>, selector: (pre: T, next: T) => boolean): Iterable<T>;
    static take<T>(items: Iterable<T>, value: number): Iterable<T>;
    static any<T>(items: Iterable<T>, selector: (elem: T) => boolean): boolean;
    static skip<T>(items: Iterable<T>, value: number): Iterable<T>;
    static order<T>(items: Iterable<T>, type: 'asc' | 'desc', selector: (elem: T) => any): T[];
    static forEach<T>(items: Iterable<T>, action: (elem: T, index: number) => boolean): void;
    static concat<T>(items: Iterable<T>[]): Iterable<T>;
    static group<TKey, T, TResult>(items: Iterable<T>, selector: (element: T) => TKey, map: (elem: T) => TResult): Iterable<Group<TKey, TResult>>;
    static lookup<TKey, T, TResult>(items: Iterable<T>, parentSelector: (elem: T) => TKey, selector: (elem: T) => TKey, map: (elem: T) => TResult): Iterable<Group<TKey, TResult>>;
    static unwind<T, TResult>(items: Iterable<T>, selector: (elem: T) => TResult[]): Iterable<TResult>;
    static unwindMap<T, TResult, TMap>(items: Iterable<T>, selector: (elem: T) => TResult[], mapSelector: (elem: TResult, parent: T) => TMap): Iterable<TMap>;
    static max<T, TResult>(items: Iterable<T>, selector?: (elem: T) => TResult): TResult;
    static min<T, TResult>(items: Iterable<T>, selector?: (elem: T) => TResult): TResult;
    static toObject<T, TResult>(items: Iterable<T>, keySelector: (elem: T) => string | number, map: (elem: T) => TResult): ObjectDic<TResult>;
}
export interface EnumerableIterator<T> extends Iterable<T> {
    array: Array<T>;
    contains(element: T): boolean;
    contains(element: T, selector: (pre: T, current: T) => boolean): boolean;
    sum(selector: (element: T) => number): number;
    filter(predicate: (elem: T) => boolean): EnumerableIterator<T>;
    map<Result>(selector: (elem: T) => Result): EnumerableIterator<Result>;
    distinct(selector: (pre: T, current: T) => boolean): EnumerableIterator<T>;
    distinct(): EnumerableIterator<T>;
    reduce(selector: (pre: T, current: T) => T): T;
    count(): number;
    take(value: number): EnumerableIterator<T>;
    any(selector: (elem: T) => boolean): boolean;
    skip(value: number): EnumerableIterator<T>;
    group<TKey>(selector: (elem: T) => TKey): EnumerableIterator<Group<TKey, T>>;
    group<TKey, TResult>(selector: (elem: T) => TKey, map: (element: T) => TResult): EnumerableIterator<Group<TKey, TResult>>;
    lookup<TParent, TKey, TResult>(parentSelector: (elem: T) => TParent, selector: (elem: T) => TKey, map: (elemet: T) => TResult): EnumerableIterator<Group<TKey, TResult>>;
    lookup<TParent, TKey>(parentSelector: (elem: T) => TParent, selector: (elem: T) => TKey): EnumerableIterator<Group<TKey, T>>;
    order(type: 'asc' | 'desc', selector: (elem: T) => any): EnumerableIterator<T>;
    order(type: 'asc' | 'desc'): EnumerableIterator<T>;
    toArray(): Array<T>;
    extract(items: Iterable<T>): EnumerableIterator<T>;
    concat(...items: Iterable<T>[]): EnumerableIterator<T>;
    append(element: T): EnumerableIterator<T>;
    forEach(action: (elem: T, index: number) => boolean): void;
    min<TResult>(selector: (elem: T) => TResult): TResult;
    max<TResult>(selector: (elem: T) => TResult): TResult;
    toObject<TResult>(keySelector: (elem: T) => string, selector: (elem: T) => TResult): ObjectDic<TResult>;
    toObject(keySelector: (elem: T) => string): ObjectDic<T>;
    unwind<T, TResult>(selector: (elem: T) => TResult[]): EnumerableIterator<TResult>;
    unwindMap<T, TResult, TMap>(selector: (elem: T) => TResult[], mapSelector: (elem: TResult, parent: T) => TMap): EnumerableIterator<TMap>;
}
export declare function empty<T>(): EnumerableIterator<T>;
export declare function enumerable<T>(iterable: Iterable<T>): EnumerableIterator<T>;
