export declare enum ExpressionType {
    binary = "binary",
    call = "call",
    property = "property",
    constant = "constant",
    sourceCall = "sourceCall",
    callChain = "callChain"
}
export declare enum Operator {
    eq = "eq",
    ne = "ne",
    lt = "lt",
    lte = "lte",
    gt = "gt",
    gte = "gte",
    or = "or",
    and = "and"
}
export interface Expression {
    type: ExpressionType;
}
export interface BinaryExpression extends Expression {
    left: Expression;
    operator: Operator;
    right: Expression;
}
export interface PropertyExpression extends Expression {
    name: string;
}
export interface ConstantExpression extends Expression {
    value: any;
}
export declare enum CallExpressionType {
    startsWith = "startsWith",
    endsWith = "endsWith",
    contains = "contains",
    equal = "equal",
    in = "in",
    any = "any",
    all = "all",
    elemMatch = "elemMatch",
    size = "size",
    exists = "exists"
}
export interface CallExpression extends Expression {
    target: Expression;
    method: CallExpressionType;
    args: Expression[];
}
export declare enum SourceCallType {
    filter = "filter",
    first = "first",
    sort = "sort",
    skip = "skip",
    take = "take",
    select = "select",
    count = "count"
}
export interface SourceCallExpression extends Expression {
    method: SourceCallType;
    args: Expression[];
}
export interface CallChainExpression extends Expression {
    expression: Expression;
    next?: Expression;
}
export interface ExpressionVisitor {
    visit(expression: {
        type: ExpressionType;
    }): any;
    binary(binary: BinaryExpression): any;
    property(property: PropertyExpression): any;
    call(call: CallExpression): any;
    constant(constant: ConstantExpression): any;
    sourceCall(sourceCall: SourceCallExpression): any;
    callChain(callChain: CallChainExpression): any;
}
export declare function visitor(): ExpressionVisitor;
