"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.StringVisitor = void 0;
const query_string_1 = require("./query-string");
const enumerable_1 = require("../enumerable");
class StringVisitor extends query_string_1.Visitor {
    constructor() {
        super();
        this.filters = [];
    }
    insert(key, value) {
        this.filters.push({ key, value });
    }
    composite(composite) {
        [composite.qr, composite.filter, composite.order, composite.page]
            .filter(x => x != null)
            .forEach((i) => {
            this.visit(i);
        });
    }
    filter(filter) {
        (filter.filters || []).forEach((i) => {
            this.visit(i);
        });
    }
    key(key) {
        this.keys = key;
    }
    eq(eq) {
        this.insert(eq.name, eq.value.toString());
    }
    range(range) {
        this.insert(range.name, `${range.begin}-${range.end}`);
    }
    or(or) {
        this.insert(or.name, or.values.join(','));
    }
    order(order) {
        this.orders = order;
    }
    qr(qr) {
        this.qrs = qr;
    }
    page(page) {
        this.pages = page;
    }
    toString() {
        let groups = enumerable_1.enumerable(this.filters).group(x => x.key).toArray();
        let filters = [];
        for (let group of groups) {
            let items = Array.from(group);
            let value = items.length > 1 ? items.map(x => x.value).join(',') : items[0].value;
            filters.push(`${group.key}:${value}`);
        }
        let qr = [];
        let left = this.keys == null ? '' : this.keys.values.join(' ');
        if (this.qrs) {
            qr.push("qr=" + this.qrs.text);
        }
        if (filters.length) {
            qr.push(`filter=${filters.join(';')}`);
        }
        if (this.orders && Array.isArray(this.orders.properties)) {
            qr.push('order=' + this.orders.properties.map(p => p.name + ":" + p.type).join(';'));
        }
        if (this.pages) {
            qr.push('page=' + this.pages.count);
        }
        return qr.length ? left + "?" + qr.join('&') : left;
    }
}
exports.StringVisitor = StringVisitor;
function default_1(query) {
    let visitor = new StringVisitor();
    visitor.visit(query);
    return visitor.toString();
}
exports.default = default_1;
