"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const query_string_1 = require("./query-string");
function isNotEmpty(raw) {
    return raw != '';
}
function trimEmpty(raw) {
    if (raw == null) {
        return null;
    }
    return raw.trim(' ');
}
function isRange(raw) {
    return new RegExp('^(([0-9\\.]+)?\\-([0-9\\.]+)?)$').test(raw);
}
function getRange(raw) {
    let indexOfSeperator = raw.indexOf('-');
    let begin = raw.substr(0, indexOfSeperator);
    let end = raw.substr(indexOfSeperator + 1, raw.length);
    return {
        begin,
        end,
        type: 'range'
    };
}
function isOr(raw) {
    return raw.indexOf(',') > 0;
}
function getOr(raw) {
    let values = raw.split(',');
    return {
        type: 'or',
        values
    };
}
function parseFilter(raw) {
    return raw.split(';')
        .filter(isNotEmpty)
        .map((elem) => {
        let [name, value] = elem.split(':').filter(isNotEmpty);
        name = trimEmpty(name);
        if (isRange(value)) {
            return Object.assign(Object.assign({}, getRange(value)), { name });
        }
        if (isOr(value)) {
            return Object.assign(Object.assign({}, getOr(value)), { name });
        }
        return {
            type: 'eq',
            name,
            value
        };
    });
}
function filter(raw, filters = []) {
    return {
        type: 'filter',
        filters: parseFilter(raw).concat(filters == null ? [] : filters)
    };
}
function order(raw) {
    return [{ properties: [], type: 'order' }]
        .concat(raw.split(';'))
        .filter(isNotEmpty)
        .reduce((o, keyWithValue) => {
        let [name, type] = keyWithValue.split(':');
        o.properties.push({
            name: trimEmpty(name),
            type
        });
        return o;
    });
}
function default_1(raw) {
    let routes = (function () {
        if (raw.indexOf('?') >= 0) {
            return raw.split('?')[0].split(' ').filter(isNotEmpty);
        }
        return raw.split(' ').filter(isNotEmpty);
    })();
    let base = { filter: {} };
    if (routes.length) {
        base.filter = {
            type: 'filter',
            filters: [{
                    type: 'key',
                    values: routes
                }]
        };
    }
    let qr = (function () {
        let i = raw.indexOf('?');
        if (i < 0) {
            return "";
        }
        return raw.substr(i + 1, raw.length);
    })();
    let q = [base]
        .concat(qr.split('&'))
        .filter(isNotEmpty)
        .reduce((o, keyWithValue) => {
        let separatorIndex = keyWithValue.indexOf('=');
        if (separatorIndex < 0) {
            return o;
        }
        let key = trimEmpty(keyWithValue.substr(0, separatorIndex));
        let value = keyWithValue.substr(separatorIndex + 1, keyWithValue.length);
        if (key === 'filter') {
            return Object.assign(Object.assign({}, o), { "filter": filter(value, (o.filter || { filters: [] }).filters) });
        }
        if (key === 'order') {
            return Object.assign(Object.assign({}, o), { "order": order(value) });
        }
        if (key === 'page') {
            return Object.assign(Object.assign({}, o), { "page": {
                    type: 'page',
                    count: parseInt(value)
                } });
        }
        if (key === 'qr') {
            return Object.assign(Object.assign({}, o), { "qr": {
                    type: 'qr',
                    text: value
                } });
        }
        return o;
    });
    return query_string_1.createQuery(q);
}
exports.default = default_1;
