export interface Element {
    type: string;
}
export interface Composite extends Element {
    type: 'composite';
    filter: Filter;
    order: Order;
    page: Page;
    qr: Qr;
}
export interface Filter extends Element {
    type: 'filter';
    filters: Array<Element>;
}
export interface Eq extends Element {
    type: 'eq';
    name: string;
    value: any;
}
export interface Range extends Element {
    type: 'range';
    name: string;
    begin: any;
    end: any;
}
export interface Or extends Element {
    type: 'or';
    name: string;
    values: Array<any>;
}
export interface Key extends Element {
    type: 'key';
    values: Array<string>;
}
export interface Order extends Element {
    type: 'order';
    properties: Array<{
        name: string;
        type: 'asc' | 'desc';
    }>;
}
export interface Page extends Element {
    type: 'page';
    count: number;
}
export interface Qr extends Element {
    type: 'qr';
    text: string;
}
export declare class Visitor {
    visit(o: Element): void;
    eq(eq: Eq): void;
    composite(composite: Composite): void;
    filter(filter: Filter): void;
    order(order: Order): void;
    page(page: Page): void;
    qr(qr: Qr): void;
    key(key: Key): void;
    range(range: Range): void;
    or(or: Or): void;
}
export interface CompositeWith extends Composite {
    eq(name: string, value: any): CompositeWith;
    range(name: string, begin: any, end: any): CompositeWith;
    contains(name: string, values: Array<any>): CompositeWith;
    any(name: string, value: any): boolean;
    exists(name: string): boolean;
    remove(name: string, value: any): CompositeWith;
    removeAll(name: string): CompositeWith;
}
export declare function createQuery(o?: Composite): CompositeWith;
