"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getSize = exports.escapeCallChain = exports.decorate = exports.queryable = exports.ArrayQueryable = void 0;
const expressions_1 = require("./expressions");
const array_visitor_1 = require("./array-visitor");
class ArrayQueryable {
    constructor(source, expression) {
        this.source = source;
        this.expression = expression;
    }
    query(...expressions) {
        let queryable = new ArrayQueryable(this.source);
        let all = this.expression == null ? expressions : [this.expression].concat(expressions);
        if (all.length === 1) {
            queryable.expression = all[0];
            return queryable;
        }
        if (all.length === 0) {
            return queryable;
        }
        let firstChain = null;
        all.map(x => ({
            type: expressions_1.ExpressionType.callChain,
            expression: x,
            next: null
        })).reduce((previous, current) => {
            if (firstChain == null) {
                firstChain = previous;
            }
            previous.next = current;
            return current;
        });
        queryable.expression = firstChain;
        return queryable;
    }
    memorize() {
        if (this[Symbol.iterator] == null) {
            return Promise.resolve(this.source);
        }
        let haveFirst = array_visitor_1.anyFirst(this.expression);
        let result = this[Symbol.iterator]();
        if (haveFirst) {
            return Promise.resolve(Array.from(result)[0]);
        }
        return Promise.resolve(Array.from(result));
    }
    [Symbol.iterator]() {
        if (this.expression == null) {
            return Array.isArray(this.source || []) ? this.source[Symbol.iterator]() : this.source;
        }
        return array_visitor_1.arrayVisitor(this.source).visit(this.expression);
    }
    cursor() {
        return this[Symbol.iterator]();
    }
}
exports.ArrayQueryable = ArrayQueryable;
function queryable(source) {
    return new ArrayQueryable(source);
}
exports.queryable = queryable;
function decorate(queryable, sets) {
    sets = sets || {};
    return {
        expression: queryable.expression,
        query: function (...expresions) {
            let newQueryable = queryable.query.apply(queryable, arguments);
            return decorate(sets.query != null ? sets.query(newQueryable) : newQueryable, sets);
        },
        memorize: function () {
            return (sets.memorize != null ? sets.memorize(queryable) : queryable.memorize());
        }
    };
}
exports.decorate = decorate;
function escapeCallChain(expressions) {
    let v = expressions_1.visitor();
    v.callChain = function (callChain) {
        let calls = [];
        if (callChain.expression != null) {
            calls.push.apply(calls, escapeCallChain([callChain.expression]));
        }
        if (callChain.next) {
            calls.push.apply(calls, escapeCallChain([callChain.next]));
        }
        return calls;
    };
    v.sourceCall = function (sourceCall) {
        let calls = [sourceCall];
        return calls;
    };
    if (expressions.length === 0)
        return [];
    let o = expressions.map(x => v.visit(x)).reduce((pre, current) => {
        return [].concat(pre, current);
    });
    let result = Array.isArray(o) ? o : [o];
    return result.filter(x => x != null);
}
exports.escapeCallChain = escapeCallChain;
function getSize(expression) {
    let v = expressions_1.visitor();
    var size = null;
    v.callChain = function (callChain) {
        if (callChain.expression) {
            v.visit(callChain.expression);
        }
        if (callChain.next) {
            v.visit(callChain.next);
        }
    };
    v.sourceCall = function (sourceCall) {
        if (sourceCall.method.toLocaleLowerCase() == "take") {
            size = sourceCall.args[0].value;
        }
    };
    v.visit(expression);
    return size;
    ;
}
exports.getSize = getSize;
