"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.escapeCallChain = exports.reduce = exports.isObject = void 0;
const enumerable_1 = require("../utility/enumerable");
const expressions_1 = require("../query/expressions");
function isObject(value) {
    if (value == null)
        return false;
    let type = typeof value;
    switch (type) {
        case 'string':
            return false;
        case 'boolean':
            return false;
        case 'function':
            return false;
        case 'number':
            return false;
        case 'undefined':
            return false;
        case 'bigint':
            return false;
        case 'symbol':
            return false;
        default:
            break;
    }
    if (value instanceof Date) {
        return false;
    }
    if (Array.isArray(value)) {
        return false;
    }
    return typeof value === 'object' && value.constructor != null && value.constructor.name === "Object";
}
exports.isObject = isObject;
function reduce(expr) {
    let expressions = escapeCallChain(expr);
    let groups = enumerable_1.group(x => x.method, expressions);
    let returns = [];
    let andAll = function (items, method) {
        return items.reduce((previous, current) => {
            return {
                type: expressions_1.ExpressionType.sourceCall,
                method,
                args: [{
                        type: expressions_1.ExpressionType.binary,
                        left: previous.args[0],
                        operator: expressions_1.Operator.and,
                        right: current.args[0]
                    }]
            };
        });
    };
    let first = groups.find(x => x.key === expressions_1.SourceCallType.first);
    if (first != null) {
        let remove = function (key) {
            let any = groups.findIndex(x => x.key === key); // remove filter
            if (any >= 0) {
                groups.splice(any, 1);
            }
        };
        let filter = groups.find(x => x.key === expressions_1.SourceCallType.filter);
        if (filter != null) {
            remove(expressions_1.SourceCallType.filter);
            first.items = [andAll([].concat(filter.items, first.items), expressions_1.SourceCallType.first)];
        }
        remove(expressions_1.SourceCallType.skip);
        remove(expressions_1.SourceCallType.sort);
        remove(expressions_1.SourceCallType.take);
    }
    groups.forEach((group) => {
        switch (group.key) {
            case expressions_1.SourceCallType.filter:
                let singleFilter = andAll(group.items, expressions_1.SourceCallType.filter);
                returns.push(singleFilter);
                break;
            case expressions_1.SourceCallType.first:
                let singleFirst = andAll(group.items, expressions_1.SourceCallType.first);
                returns.push(singleFirst);
                break;
            default:
                returns.push(last(group.items));
        }
    });
    return returns;
}
exports.reduce = reduce;
function last(items) {
    let current;
    for (let item of items) {
        current = item;
    }
    return current;
}
function escapeCallChain(expressions) {
    let v = expressions_1.visitor();
    v.callChain = function (callChain) {
        let calls = [];
        if (callChain.expression != null) {
            calls.push.apply(calls, escapeCallChain([callChain.expression]));
        }
        if (callChain.next) {
            calls.push.apply(calls, escapeCallChain([callChain.next]));
        }
        return calls;
    };
    v.sourceCall = function (sourceCall) {
        let calls = [sourceCall];
        return calls;
    };
    if (expressions.length === 0)
        return [];
    let o = expressions.map(x => v.visit(x)).reduce((pre, current) => {
        return [].concat(pre, current);
    });
    let result = Array.isArray(o) ? o : [o];
    return result.filter(x => x != null);
}
exports.escapeCallChain = escapeCallChain;
