"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.queryable = exports.expressionToYubiQlVisitor = exports.pankirin = exports.select = exports.count = exports.take = exports.skip = exports.sort = exports.first = exports.filter = void 0;
const expressions_1 = require("./expressions");
const utility_1 = require("./utility");
let expre = {
    binary: function (left, op, right) {
        return {
            type: expressions_1.ExpressionType.binary,
            left,
            operator: op,
            right
        };
    },
    constant: function (value) {
        return {
            type: expressions_1.ExpressionType.constant,
            value
        };
    },
    property: function (name) {
        return {
            type: expressions_1.ExpressionType.property,
            name
        };
    },
    call: function (target, method, args) {
        return {
            type: expressions_1.ExpressionType.call,
            target,
            method,
            args
        };
    },
    binaryManager: function (type = 'and') {
        let binary;
        return {
            add: function (newBinary) {
                if (binary == null) {
                    binary = newBinary;
                    return;
                }
                let operator = (function () {
                    if (type === 'or')
                        return expressions_1.Operator.or;
                    return expressions_1.Operator.and;
                })();
                binary = expre.binary(binary, operator, newBinary);
            },
            get: function () {
                return binary;
            }
        };
    }
};
function filter(query) {
    let commandBinary = function (key, value) {
        let binary = expre.binaryManager(key === '$or' ? 'or' : 'and');
        value.forEach((element) => binary.add(getBinary(element)));
        return binary.get();
    };
    let objectRightBinary = function (left, right) {
        let binary = expre.binaryManager();
        let prop = typeof left === 'string' ? expre.property(left) : left;
        if (!utility_1.isObject(right))
            return expre.binary(prop, expressions_1.Operator.eq, expre.constant(right));
        Object.keys(right).forEach((key) => {
            let addBinary = function (op) {
                let valueExpression = expre.constant(right[key]);
                binary.add(expre.binary(prop, op, valueExpression));
            };
            let addCall = function (method) {
                let canMoreThanArgs = [expressions_1.CallExpressionType.contains, expressions_1.CallExpressionType.startsWith, expressions_1.CallExpressionType.endsWith, expressions_1.CallExpressionType.equal];
                let args = right[key];
                let valueExpressions = canMoreThanArgs.indexOf(method) >= 0 && Array.isArray(args) ? args.map(x => expre.constant(x)) : [expre.constant(args)];
                binary.add(expre.call(prop, method, valueExpressions));
            };
            let addElemCall = function (method) {
                let binaryOfelem = getBinary(right[key]);
                binary.add(expre.call(prop, method, [binaryOfelem]));
            };
            switch (key) {
                case '$eq':
                    addBinary(expressions_1.Operator.eq);
                    return true;
                case '$ne':
                    addBinary(expressions_1.Operator.ne);
                    return true;
                case '$lt':
                    addBinary(expressions_1.Operator.lt);
                    return true;
                case '$lte':
                    addBinary(expressions_1.Operator.lte);
                    return true;
                case '$gt':
                    addBinary(expressions_1.Operator.gt);
                    return true;
                case '$gte':
                    addBinary(expressions_1.Operator.gte);
                    return true;
                case '$in':
                    addCall(expressions_1.CallExpressionType.in);
                    return true;
                case '$all':
                    addCall(expressions_1.CallExpressionType.all);
                    return true;
                case '$any':
                    addCall(expressions_1.CallExpressionType.any);
                    return true;
                case '$contains':
                    addCall(expressions_1.CallExpressionType.contains);
                    return true;
                case '$equal':
                    addCall(expressions_1.CallExpressionType.equal);
                    return true;
                case '$startsWith':
                    addCall(expressions_1.CallExpressionType.startsWith);
                    return true;
                case '$endsWith':
                    addCall(expressions_1.CallExpressionType.endsWith);
                    return true;
                case '$elemMatch':
                    addElemCall(expressions_1.CallExpressionType.elemMatch);
                    return true;
                case '$size':
                    addCall(expressions_1.CallExpressionType.size);
                    return true;
                case '$exists':
                    addCall(expressions_1.CallExpressionType.exists);
                    return true;
                default:
                    return true;
            }
        });
        return binary.get();
    };
    let getBinary = function (req, type = 'and') {
        let binary = expre.binaryManager(type);
        Object.keys(req).forEach((key) => {
            let isCommand = key.startsWith('$');
            if (isCommand) {
                binary.add(commandBinary(key, req[key]));
                return true;
            }
            let value = req[key];
            if (!utility_1.isObject(value)) {
                binary.add(expre.binary(expre.property(key), expressions_1.Operator.eq, expre.constant(value)));
                return true;
            }
            binary.add(objectRightBinary(key, req[key]));
            return true;
        });
        return binary.get();
    };
    return {
        type: expressions_1.ExpressionType.sourceCall,
        method: expressions_1.SourceCallType.filter,
        args: [getBinary(query)]
    };
}
exports.filter = filter;
function first(query) {
    let result = filter(query);
    result.method = expressions_1.SourceCallType.first;
    return result;
}
exports.first = first;
function sort(query) {
    let types = ['asc', 'desc'];
    let isType = function (type) {
        return types.indexOf(type) >= 0;
    };
    let arg = {};
    Object.keys(query).forEach((key) => {
        let type = query[key];
        if (!isType(type)) {
            throw new Error(type + " is invalid type. valid types :[" + types.join(',') + "]");
        }
        arg[key] = type;
    });
    return {
        type: expressions_1.ExpressionType.sourceCall,
        method: expressions_1.SourceCallType.sort,
        args: [expre.constant(arg)]
    };
}
exports.sort = sort;
function skip(value) {
    return {
        type: expressions_1.ExpressionType.sourceCall,
        method: expressions_1.SourceCallType.skip,
        args: [expre.constant(value)]
    };
}
exports.skip = skip;
function take(value) {
    return {
        type: expressions_1.ExpressionType.sourceCall,
        method: expressions_1.SourceCallType.take,
        args: [expre.constant(value)]
    };
}
exports.take = take;
function count() {
    return {
        type: expressions_1.ExpressionType.sourceCall,
        method: expressions_1.SourceCallType.count,
        args: []
    };
}
exports.count = count;
function select(props) {
    let o = {};
    if (Array.isArray(props)) {
        props.forEach((prop) => {
            o[prop] = 1;
        });
    }
    else {
        o = props;
    }
    return {
        type: expressions_1.ExpressionType.sourceCall,
        method: expressions_1.SourceCallType.select,
        args: [expre.constant(o)]
    };
}
exports.select = select;
function pankirinVisitor() {
    let v = expressions_1.visitor();
    v.binary = function (binary) {
        let left = pankirinVisitor().visit(binary.left);
        let right = pankirinVisitor().visit(binary.right);
        return "(" + left + " " + binary.operator + " " + right + ")";
    };
    v.constant = function (constant) {
        let type = typeof constant.value;
        let value = constant.value;
        switch (type) {
            case 'string':
                return "'" + value + "'";
            case 'number':
                return value;
            case 'boolean':
                return value;
            case 'undefined':
                return 'null';
            case 'bigint':
                return value;
            default:
                if (value instanceof Date) {
                    return "d'" + value.toISOString() + "'";
                }
                if (Array.isArray(value)) {
                    return "[" + value.join(',') + "]";
                }
                throw new Error('value could not convert');
        }
    };
    v.property = function (property) {
        return property.name;
    };
    v.call = function (call) {
        let args = [pankirinVisitor().visit(call.target)].concat(call.args.map(x => pankirinVisitor().visit(x)));
        return call.method + "(" + args.join(',') + ")";
    };
    v.callChain = function (callChain) {
        let items = [];
        let current = pankirinVisitor().visit(callChain.expression);
        items.push(current);
        if (callChain.next != null) {
            items.push(pankirinVisitor().visit(callChain.next));
        }
        return items.join('&');
    };
    v.sourceCall = function (sourceCall) {
        let args = sourceCall.args.map(x => pankirinVisitor().visit(x));
        return sourceCall.method + "=" + args.join('|');
    };
    return v;
}
function pankirin(...expressions) {
    if (expressions.length == 1) {
        return pankirinVisitor().visit(expressions[0]);
    }
    let expression = expressions.reduce((previous, current) => {
        return {
            type: expressions_1.ExpressionType.callChain,
            expression: previous,
            next: current
        };
    });
    return pankirinVisitor().visit(expression);
}
exports.pankirin = pankirin;
function expressionToYubiQlVisitor() {
    let v = expressions_1.visitor();
    v.property = function (property) {
        return property.name;
    };
    v.constant = function (constant) {
        return constant.value;
    };
    v.callChain = function (callChain) {
        let arrs = utility_1.escapeCallChain([callChain]);
        let expressions = utility_1.reduce(arrs);
        return [{}].concat(expressions.map(x => expressionToYubiQlVisitor().visit(x))).reduce((pre, current) => {
            return Object.assign(Object.assign({}, pre), current);
        });
        /*
        let o = [];
        let current = callChain as any;
        while (current != null) {
            o.push(expressionToYubiQlVisitor().visit(current.expression));
            current = current.next;
        }
        if (o.length == 0) return {};
        if (o.length === 1) return o[0];
        return [{}].concat(o).reduce((pre, current) => ({ ...pre, ...current }));
        */
    };
    v.sourceCall = function (sourceCall) {
        let o = sourceCall.args.map(a => (expressionToYubiQlVisitor().visit(a)));
        let structs = o.filter(x => !utility_1.isObject(x));
        let right;
        if (structs.length != 0) {
            if (structs.length === 1) {
                right = structs[0];
            }
            else {
                right = structs;
            }
        }
        else {
            right = o.length == 0 ? {} : [{}].concat(o)
                .reduce((pre, current) => (Object.assign(Object.assign({}, pre), current)));
        }
        return {
            ["$" + sourceCall.method]: right
        };
    };
    v.binary = function (binary) {
        let left = expressionToYubiQlVisitor().visit(binary.left);
        let right = expressionToYubiQlVisitor().visit(binary.right);
        let bind = function (op) {
            return { [left]: { [op]: right } };
        };
        switch (binary.operator) {
            case expressions_1.Operator.and:
                return { $and: [left, right] };
            case expressions_1.Operator.eq:
                return { [left]: right };
            case expressions_1.Operator.ne:
                return bind('$ne');
            case expressions_1.Operator.or:
                return (function () {
                    return { "$or": [left, right] };
                })();
            case expressions_1.Operator.gte:
                return bind('$gte');
            case expressions_1.Operator.gt:
                return bind('$gt');
            case expressions_1.Operator.lte:
                return bind('$lte');
            case expressions_1.Operator.lt:
                return bind('$lt');
            default:
                throw new Error(binary.operator + " cant find");
        }
    };
    v.call = function (call) {
        let args = call.args.map(a => {
            return expressionToYubiQlVisitor().visit(a);
        });
        let o = args.length === 1 ? args[0] : args;
        let method = call.method.toString();
        let value = o;
        let right = { ["$" + method]: value };
        let left = expressionToYubiQlVisitor().visit(call.target);
        return { [left]: right };
    };
    return v;
}
exports.expressionToYubiQlVisitor = expressionToYubiQlVisitor;
function queryable(done) {
    return {
        expression: null,
        query(...expressions) {
            let qr = queryable(done);
            let all = this.expression == null ? expressions : [this.expression].concat(expressions);
            if (all.length === 1) {
                qr.expression = all[0];
                return qr;
            }
            qr.expression = all.reverse().reduce((previous, current) => {
                return {
                    type: 'callChain',
                    expression: current,
                    next: previous.type == 'callChain' ? previous : {
                        type: 'callChain',
                        expression: previous
                    }
                };
            });
            return qr;
        },
        toJson() {
            let q = {};
            if (this.expression !== null) {
                q = expressionToYubiQlVisitor().visit(this.expression);
            }
            return q;
        },
        memorize() {
            return done(this.toJson());
        }
    };
}
exports.queryable = queryable;
