export declare type ValidatorFunction = (e: ValidationModel, i: IteratorArg) => any;
export declare type ValueConvertFunction = (context: ValueContext) => any;
export interface Model {
    [key: string]: PropertyOptions;
}
export interface PropertyOptions {
    type: any;
    element?: any;
    validators?: Array<ValidatorFunction>;
    validate?: boolean;
    default?: any;
    enum?: Array<any>;
}
export interface ValueContext {
    property: string;
    value: any;
    type: any;
    model: PropertyOptions;
    schema: PropertyOptions;
    options?: any;
}
export interface ModelOptions {
    converters: Array<ValueConvertFunction>;
    validators: Array<ValidatorFunction>;
    validation?: boolean;
    enum?: Array<any>;
    partial?: boolean;
    escape?: Array<string>;
}
export interface ValidationModel {
    model: PropertyOptions;
    schema: PropertyOptions;
    prop: string;
    value: any;
    type: any;
}
export declare type IteratorArg = {
    done: (e: any) => void;
    next: () => void;
};
export interface ValidationResult {
    isValid: boolean;
    message?: string;
}
export declare type ValidatedModelResult = {
    value: any;
    isValid: boolean;
    error: Object;
};
export declare type ModelContext = {
    data: any;
    model: PropertyOptions;
    options: ModelOptions;
};
export interface ValidatorVisitor {
    any(modelContext: ModelContext): ValidatedModelResult;
    array(array: ArrayDefined): ValidatedModelResult;
    object(object: ObjectDefined): ValidatedModelResult;
    primitive(primitive: PrimitiveDefined): ValidatedModelResult;
}
export interface ModelDefined {
    type: any;
    value: any;
    model: PropertyOptions;
    property?: string;
    options: ModelOptions;
}
export interface ObjectDefined extends ModelDefined {
}
export interface PrimitiveDefined extends ModelDefined {
}
export interface ArrayDefined extends ModelDefined {
    element: any;
    elementType: any;
}
