"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Any = exports.serialize = exports.dateConverter = exports.dateNow = exports.forModel = exports.getOptions = exports.parseType = exports.converters = exports.validate = exports.validators = exports.schema = exports.type = void 0;
const utility_1 = require("./utility");
const validation_1 = require("./validation");
exports.type = '@type';
exports.schema = 'schema';
exports.validators = 'validators';
exports.validate = 'validate';
exports.converters = 'converters';
class ModelValidator {
    any(context) {
        if (context == null || context.model == null)
            throw new Error('context is null');
        let { data, model, options, property } = context;
        if (options.validation === false) {
            return {
                isValid: true,
                value: data,
                error: {}
            };
        }
        let pt = parseType(model);
        let theModel = getSchema(model);
        let value = convert(model, theModel, pt.type || Object, property || '__model__', data, options.converters, options);
        let isPropertyMustValidate = isValidate(model);
        if (!isPropertyMustValidate) {
            return {
                isValid: true,
                value,
                error: {}
            };
        }
        let propertyValidation = (function () {
            let validatorsAll = [...getValidators(model), ...(options.validators || [])];
            let validationResult = null;
            utility_1.toNextIterator(validatorsAll, { model, schema: theModel, prop: property || '__model__', value, type: pt.type || Object }, function (e) {
                validationResult = e;
            });
            if (validationResult != null && !validationResult.isValid) { // property is not valid
                return { result: false, error: validationResult.message };
            }
            return { result: true, break: validationResult != null ? validationResult.break || false : false };
        })();
        if (!propertyValidation.result)
            return {
                isValid: false,
                value,
                error: propertyValidation.error
            };
        if (propertyValidation.break) {
            return {
                value,
                isValid: true,
                error: {}
            };
        }
        if (pt.is(Array)) {
            let getElement = function () {
                if (Array.isArray(model)) {
                    return model[0];
                }
                let propertyOptions = model;
                if (propertyOptions[exports.type] != null) {
                    return propertyOptions.element;
                }
                return null;
            };
            return this.array({
                type: pt.type,
                value,
                model: model,
                options: options,
                property: property,
                element: getElement(),
                elementType: (function () {
                    return getType(getElement());
                })()
            });
        }
        if (pt.is(Object)) {
            return this.object({
                type: pt.type,
                value,
                property: property,
                model: theModel,
                options: options
            });
        }
        return this.primitive({
            type: pt.type,
            property: property,
            value,
            model: theModel,
            options: options
        });
    }
    array(array) {
        let { value, element, elementType, options } = array;
        let error = {};
        let result = value.map((item, index) => convert(element, element, elementType, index, item, options.converters, options));
        let validationResults = result.map((data, index) => {
            return {
                result: this.any({ data, model: element, options }),
                element,
                index
            };
        });
        let errors = validationResults.filter(x => !x.result.isValid);
        if (errors.length != 0) {
            errors.forEach((validationResult) => {
                error = Object.assign(Object.assign({}, error), { [validationResult.index]: validationResult.result.error });
            });
        }
        let isValid = errors.length === 0;
        return {
            value: isValid ? validationResults.map(x => x.result.value) : result,
            error,
            isValid
        };
    }
    object(object) {
        let { value, model, options } = object;
        value = value || {};
        if (model == null)
            return {
                value,
                isValid: true,
                error: {}
            };
        let keys = options.partial == true ? (function () {
            let o = Object.keys(model);
            return Object.keys(value || {}).filter(key => o.indexOf(key) >= 0);
        })() : Object.keys(model);
        let errors = [];
        let result = {};
        for (let key of keys) {
            let validationResult = this.any({ data: value[key], model: model[key], options, property: key });
            if (!validationResult.isValid) {
                errors.push(mergeParent(key, validationResult.error));
                continue;
            }
            result[key] = validationResult.value;
        }
        return {
            isValid: errors.length === 0,
            value: result,
            error: errors.length == 0 ? {} : errors.reduce((pre, current) => {
                return Object.assign(Object.assign({}, pre), current);
            })
        };
    }
    primitive(primitive) {
        let { value } = primitive;
        return {
            value,
            isValid: true,
            error: {}
        };
    }
}
//const rezervedProps = ['type', 'validators', "validate",  "default","enum"];
function getType(typable) {
    if (typable == null)
        return undefined;
    if (typeof typable === "function")
        return typable; // name:String
    if (Array.isArray(typable)) { // items:[{String},{type:Number}];
        return Array;
    }
    if (utility_1.checkObject(typable)) { // item :{}
        if (typeof typable[exports.type] === "function") {
            return getType(typable[exports.type]);
        }
        return Object; // item {name:String,value:Number};
    }
    return undefined;
}
function parseType(typable) {
    let type = getType(typable);
    return {
        type,
        is: function (t) {
            return this.type === t;
        }
    };
}
exports.parseType = parseType;
function convert(propModel, schema, type, key, value, converters, options) {
    let result = value;
    converters.forEach((converter) => {
        result = converter({ value: result, schema, type, property: key, model: propModel, options });
    });
    return result;
}
function mergeParent(name, o) {
    if (typeof o === "string") {
        return { [name]: o };
    }
    let result = {};
    result[name] = {};
    Object.keys(o).forEach((key) => {
        result[name][key] = o[key];
    });
    return result;
}
function getOptions() {
    return {
        converters: [onlyModelProperty, defaultConverter, dateConverter],
        validators: [validation_1.strict, validation_1.typeCheck, validation_1.enumCheck],
        validation: true
    };
}
exports.getOptions = getOptions;
function forModel(model, options) {
    return {
        validate: function (data) {
            let visitor = new ModelValidator();
            return visitor.any({ data, model: model, options: options || getOptions() });
        }
    };
}
exports.forModel = forModel;
function onlyModelProperty(context) {
    let { property, schema, value, type, model, options } = context;
    if (value == null)
        return null;
    if (type === Object) {
        if (options == null || options.partial == null || options.partial == false) {
            let o = {};
            Object.keys(schema).forEach((key) => {
                o[key] = value[key];
            });
            return o;
        }
        if (options != null && options.partial == true) {
            let o = {};
            let keys = Object.keys(schema);
            Object.keys(value).forEach((key) => {
                if (keys.indexOf(key) < 0)
                    return true;
                o[key] = value[key];
            });
            return o;
        }
    }
    return value;
}
function getSchema(model) {
    if (model == null)
        return null;
    if (model[exports.type]) {
        return model[exports.schema];
    }
    return model;
}
function getConverters(model) {
    if (model == null)
        return [];
    if (model[exports.type]) {
        return model[exports.converters] || [];
    }
    return [];
}
function getValidators(model) {
    if (model == null)
        return [];
    if (model[exports.type]) {
        return model[exports.validators] || [];
    }
    return [];
}
function isValidate(model) {
    if (model == null)
        return true;
    if (model[exports.type]) {
        return model[exports.validate] || true;
    }
    return true;
}
function defaultConverter(context) {
    if (context.value != null)
        return context.value;
    if (context.type !== Object && (context.model || {}).default != null) {
        const value = context.model.default;
        if (typeof value === "function") {
            return value(context);
        }
        return value;
    }
    return context.value;
}
function dateNow() {
    return function () {
        return new Date(Date.now());
    };
}
exports.dateNow = dateNow;
function dateConverter(context) {
    if (context.type === Date) {
        if (context.value instanceof Date) {
            return context.value;
        }
        if (context.value == null) {
            return context.value;
        }
        var result = new Date(context.value);
        if (isNaN(result.getTime())) {
            return null;
        }
        return result;
    }
    return context.value;
}
exports.dateConverter = dateConverter;
function serialize(model) {
    let pt = parseType(model);
    if (pt.is(Object)) {
        let o = {};
        Object.keys(model).forEach((key) => {
            o[key] = serialize(model[key]);
        });
        return o;
    }
    if (pt.is(Array)) {
        let o = [];
        if (Array.isArray(model)) {
            model.forEach((element) => {
                o.push(serialize(element));
            });
        }
        else if (typeof model === "object") {
            o.push(serialize(model.element || {}));
        }
        return o;
    }
    if (typeof pt.type === "function") {
        if (model.enum != null)
            return model.enum.join('|');
        return pt.type.name;
    }
}
exports.serialize = serialize;
class Any {
    constructor() {
        this[exports.type] = 'any';
    }
}
exports.Any = Any;
