"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.greaterThan = exports.positive = exports.enumCheck = exports.notNullOrEmpty = exports.emptyStringToNull = exports.notEmpty = exports.required = exports.typeCheck = exports.strict = exports.nullable = void 0;
const model_1 = require("./model");
function error(message) {
    return {
        isValid: false,
        message: message
    };
}
function nullable(e, i) {
    if (e.value == null) {
        return i.done({
            isValid: true,
            message: 'ok',
            break: true
        });
    }
    return i.next();
}
exports.nullable = nullable;
function strict(e, i) {
    if (e.type == Object) {
        let keys = Object.keys(e.schema || {});
        let valueKeys = Object.keys(e.value || {});
        let extraKeys = valueKeys.filter(key => keys.indexOf(key) < 0);
        return i.done({ isValid: extraKeys.length == 0, message: extraKeys.join(',') + " props should not be in the model" });
    }
    return i.next();
}
exports.strict = strict;
function typeCheck(e, i) {
    if (e.type == null)
        return i.next();
    if (e.value != null && isSameType(e.value.__proto__.constructor, e.type))
        return i.next();
    if (e.type == model_1.Any || (e.type.name === "Any" && e.value)) {
        return i.next();
    }
    return i.done(error('field type must ' + (e.type || {}).name));
}
exports.typeCheck = typeCheck;
function isSameType(left, right) {
    function toLower(v) {
        if (v == null)
            return null;
        return v.toLowerCase();
    }
    let notNull = left.name != null && right.name != null;
    let nameEq = toLower(left.name) == toLower(right.name);
    let eq = left == right;
    let result = notNull && (nameEq || eq);
    return result;
}
function required(e, i) {
    if (e.value == null)
        return i.done(error('field is required'));
    return i.next();
}
exports.required = required;
function notEmpty(e, i) {
    if (e.value == '')
        return i.done(error('could not empty'));
    return i.next();
}
exports.notEmpty = notEmpty;
function emptyStringToNull(e, i) {
    if (e.value == '')
        return null;
    return i.next();
}
exports.emptyStringToNull = emptyStringToNull;
function notNullOrEmpty(e, i) {
    if (e.value == null || e.value == '')
        return i.done(error('field must not null or empty'));
    return i.next();
}
exports.notNullOrEmpty = notNullOrEmpty;
function enumCheck(e, i) {
    if (e.type !== Object && e.model.enum) {
        let inEnum = e.model.enum.some(x => e.value === x);
        if (inEnum)
            return i.next();
        return i.done(error('field\'s value must only ' + e.model.enum.join('|')));
    }
}
exports.enumCheck = enumCheck;
function positive(e, i) {
    let value = e.value;
    if (typeof value !== "number" || value < 0) {
        return i.done(error('field must positive'));
    }
    return i.next();
}
exports.positive = positive;
function greaterThan(value) {
    return function (e, i) {
        let v = e.value;
        if (typeof v !== "number" || v > value) {
            return i.done(error('value must greater than ' + value));
        }
        return i.next();
    };
}
exports.greaterThan = greaterThan;
